# -*- coding:utf-8 -*-
"""
Created on 2015/02/04
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""
import pandas as pd
import tushare as ts
from pandas import compat
import os


class Store(object):

    def __init__(self, data=None, name=None, path=None):
        if isinstance(data, pd.DataFrame):
            self.data = data
        else:
            raise RuntimeError('data type is incorrect')
        self.name = name
        self.path = path

    def save_as(self, name, path, to='csv'):
        if name is None:
            name = self.name
        if path is None:
            path = self.path
        file_path = '%s%s%s.%s'
        if isinstance(name, compat.string_types) and name is not '':
            if (path is None) or (path == ''):
                file_path = '.'.join([name, to])
            else:
                try:
                    if os.path.exists(path) is False:
                        os.mkdir(path) 
                    file_path = file_path%(path, '/', name, to)
                except:
                    pass
            
        else:
            print('input error')




# -*- coding:utf-8 -*- 
"""
通联数据
Created on 2015/08/24
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""

from pandas.compat import StringIO
import pandas as pd
from tushare.util import vars as vs
from tushare.util.common import Client
from tushare.util import upass as up

class Equity():
    
    def __init__(self, client=None):
        if client is None:
            self.client = Client(up.get_token())
        else:
            self.client = client
        
    def Equ(self, equTypeCD='', secID='', ticker='', listStatusCD='', field=''):
        """
            获取股票的基本信息，包含股票交易代码及其简称、股票类型、上市状态、上市板块、上市日期等；上市状态为最新数据，不显示历史变动信息。
        """
        code, result = self.client.getData(vs.EQU%(equTypeCD, secID, ticker, listStatusCD, field))
        return _ret_data(code, result)


    def EquAllot(self, isAllotment='', secID='', ticker='', beginDate='', endDate='', field=''):
        """
            获取股票历次配股的基本信息，包含每次配股方案的内容、方案进度、历史配股预案公布次数以及最终是否配股成功。
        """
        code, result = self.client.getData(vs.EQUALLOT%(isAllotment, secID, ticker, 
                                                        beginDate, endDate, field))
        return _ret_data(code, result)


    def EquDiv(self, eventProcessCD='', exDivDate='', secID='', ticker='', beginDate='', 
               endDate='', field=''):
        """
            获取股票历次分红(派现、送股、转增股)的基本信息，包含历次分红预案的内容、实施进展情况以及历史宣告分红次数。
        """
        code, result = self.client.getData(vs.EQUDIV%(eventProcessCD, exDivDate, 
                                                      secID, ticker, beginDate, endDate, field))
        return _ret_data(code, result)


    def EquIndustry(self, industry='', industryID='', industryVersionCD='', secID='', 
                    ticker='', intoDate='', field=''):
        """
            输入证券ID或股票交易代码，获取股票所属行业分类
        """
        code, result = self.client.getData(vs.EQUINDUSTRY%(industry, industryID, industryVersionCD, 
                                                           secID, ticker, intoDate, field))
        return _ret_data(code, result)


    def EquIPO(self, eventProcessCD='', secID='', ticker='', field=''):
        """
            获取股票首次公开发行上市的基本信息，包含股票首次公开发行的进程及发行结果。
        """
        code, result = self.client.getData(vs.EQUIPO%(eventProcessCD, secID, ticker, field))
        return _ret_data(code, result)


    def EquRef(self, secID='', ticker='', beginDate='', endDate='', eventProcessCD='', field=''):
        """
            获取股票股权分置改革的基本信息，包含股改进程、股改实施方案以及流通股的变动情况。
        """
        code, result = self.client.getData(vs.EQUREF%(secID, ticker, beginDate, endDate, 
                                                      eventProcessCD, field))
        return _ret_data(code, result)


    def EquRetud(self, listStatusCD='', secID='', ticker='', beginDate='', 
                 dailyReturnNoReinvLower='', dailyReturnNoReinvUpper='', 
                 dailyReturnReinvLower='', dailyReturnReinvUpper='', 
                 endDate='', isChgPctl='', field=''):
        """
            获取股票每日回报率的基本信息，包含交易当天的上市状态、日行情以及除权除息事项的基本数据。
        """
        code, result = self.client.getData(vs.EQURETUD%(listStatusCD, secID, ticker, 
                                                        beginDate, dailyReturnNoReinvLower, 
                                                        dailyReturnNoReinvUpper, 
                                                        dailyReturnReinvLower, 
                                                        dailyReturnReinvUpper, 
                                                        endDate, isChgPctl, field))
        return _ret_data(code, result)


    def EquSplits(self, secID='', ticker='', beginDate='', endDate='', field=''):
        """
            获取股票进行股本拆细或者缩股的基本信息。
        """
        code, result = self.client.getData(vs.EQUSPLITS%(secID, ticker, beginDate, 
                                                         endDate, field))
        return _ret_data(code, result)


    def FstTotal(self, beginDate='', endDate='', exchangeCD='', field=''):
        """
            获取上海、深圳交易所公布的每个交易日的融资融券交易汇总的信息，包括成交量、成交金额。本交易日可获取前一交易日的数据。
        """
        code, result = self.client.getData(vs.FSTTOTAL%(beginDate, endDate, 
                                                        exchangeCD, field))
        return _ret_data(code, result)


    def FstDetail(self, secID='', ticker='', beginDate='', endDate='', field=''):
        """
            获取上海、深圳交易所公布的每个交易日的融资融券交易具体的信息，包括标的证券信息、融资融券金额以及数量方面的数据。本交易日可获取前一交易日的数据。
        """
        code, result = self.client.getData(vs.FSTDETAIL%(secID, ticker, 
                                                         beginDate, endDate, field))
        return _ret_data(code, result)


    def EquShare(self, secID='', ticker='', beginDate='', endDate='', 
                 partyID='', field=''):
        """
            获取上市公司股本结构及历次股本变动数据。
        """
        code, result = self.client.getData(vs.EQUSHARE%(secID, ticker, 
                                                        beginDate, endDate, 
                                                        partyID, field))
        return _ret_data(code, result)


    def SecST(self, secID='', ticker='', beginDate='', endDate='', field=''):
        """
            通过输入股票ID（通联编制）或股票交易代码（支持多值输入，最大支持50只），选择查询开始日期与结束日期，获取股票在一段时间ST标记信息。
        """
        code, result = self.client.getData(vs.SECST%(secID, ticker, 
                                                     beginDate, endDate, field))
        return _ret_data(code, result)


def _ret_data(code, result):
    if code==200:
        result = result.decode('utf-8') if vs.PY3 else result
        df = pd.read_csv(StringIO(result))
        return df
    else:
        print(result)
        return None    




# -*- coding:utf-8 -*- 
"""
通联数据
Created on 2015/08/24
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""

from pandas.compat import StringIO
import pandas as pd
from tushare.util import vars as vs
from tushare.util.common import Client
from tushare.util import upass as up

class Future():
    
    def __init__(self, client=None):
        if client is None:
            self.client = Client(up.get_token())
        else:
            self.client = client
            
            
    def Futu(self, exchangeCD='', secID='', ticker='', contractObject='', field=''):
        """
            获取国内四大期货交易所期货合约的基本要素信息，
            包括合约名称、合约代码、合约类型、合约标的、报价单位、最小变动价位、涨跌停板幅度、交易货币、
            合约乘数、交易保证金、上市日期、最后交易日、交割日期、交割方式、交易手续费、交割手续费、挂牌基准价、合约状态等。
        """
        code, result = self.client.getData(vs.FUTU%(exchangeCD, secID, ticker, contractObject, field))
        return _ret_data(code, result)


    def FutuConvf(self, secID='', ticker='', field=''):
        """
            获取国债期货转换因子信息，包括合约可交割国债名称、可交割国债交易代码、转换因子等。
        """
        code, result = self.client.getData(vs.FUTUCONVF%(secID, ticker, field))
        return _ret_data(code, result)


def _ret_data(code, result):
    if code==200:
        result = result.decode('utf-8') if vs.PY3 else result
        df = pd.read_csv(StringIO(result))
        return df
    else:
        print(result)
        return None    
    



# -*- coding:utf-8 -*- 
"""
通联数据
Created on 2015/08/24
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""

from pandas.compat import StringIO
import pandas as pd
from tushare.util import vars as vs
from tushare.util.common import Client
from tushare.util import upass as up

class Master():
    
    def __init__(self, client=None):
        if client is None:
            self.client = Client(up.get_token())
        else:
            self.client = client
            
            
    def SecID(self, assetClass='', cnSpell='', partyID='', ticker='', field=''):
        """
            通过机构partyID,或证券简称拼音cnSpell,或证券交易代码ticker，
            检索证券ID（证券在数据结构中的一个唯一识别的编码），
            也可通过证券简称拼音cnSpell检索证券交易代码ticker等；同时可以获取输入证券的基本上市信息，如交易市场，上市状态，交易币种，ISIN编码等。
        """
        code, result = self.client.getData(vs.SECID%(assetClass, cnSpell, partyID, ticker, field))
        return _ret_data(code, result)
    

    def TradeCal(self, exchangeCD='', beginDate='', endDate='', field=''):
        """
            记录了上海证券交易所,深圳证券交易所,中国银行间市场,大连商品交易所,郑州商品交易所,上海期货交易所,
            中国金融期货交易所和香港交易所等交易所在日历日期当天是否开市的信息，
            其中上证、深证记录了自成立以来的全部日期是否开始信息。各交易日节假日安排通知发布当天即更新数据。
        """
        code, result = self.client.getData(vs.TRADECAL%(exchangeCD, beginDate, endDate, field))
        return _ret_data(code, result)


    def Industry(self, industryVersion='', industryVersionCD='', industryLevel='', isNew='', field=''):
        """
            输入行业分类通联编码(如，010303表示申万行业分类2014版)或输入一个行业分类标准名称，获取行业分类标准下行业划分
        """
        code, result = self.client.getData(vs.INDUSTRY%(industryVersion, industryVersionCD,
                                                        industryLevel, isNew, field))
        return _ret_data(code, result)


    def SecTypeRel(self, secID='', ticker='', typeID='', field=''):
        """
            记录证券每个分类的成分，证券分类可通过在getSecType获取。
        """
        code, result = self.client.getData(vs.SECTYPEREL%(secID, ticker, typeID, field))
        return _ret_data(code, result)


    def EquInfo(self, ticker='', field=''):
        """
            根据拼音或股票代码，匹配股票代码、名称。包含正在上市的全部沪深股票。
        """
        code, result = self.client.getData(vs.EQUINFO%(ticker, field))
        return _ret_data(code, result)


    def SecTypeRegionRel(self, secID='', ticker='', typeID='', field=''):
        """
            获取沪深股票地域分类，以注册地所在行政区域为标准。
        """
        code, result = self.client.getData(vs.SECTYPEREGIONREL%(secID, ticker, typeID, field))
        return _ret_data(code, result)


    def SecType(self, field=''):
        """
            证券分类列表，一级分类包含有沪深股票、港股、基金、债券、期货、期权等，每个分类又细分有不同类型；可一次获取全部分类。
        """
        code, result = self.client.getData(vs.SECTYPE%(field))
        return _ret_data(code, result)


    def SecTypeRegion(self, field=''):
        """
            获取中国地域分类，以行政划分为标准。
        """
        code, result = self.client.getData(vs.SECTYPEREGION%(field))
        return _ret_data(code, result)


    def SysCode(self, codeTypeID='', valueCD='', field=''):
        """
            各api接口有枚举值特性的输出列，如getSecID输出项exchangeCD值，编码分别代表的是什么市场，所有枚举值都可以在这个接口获取。
        """
        code, result = self.client.getData(vs.SYSCODE%(codeTypeID, valueCD, field))
        return _ret_data(code, result)


def _ret_data(code, result):
    if code==200:
        result = result.decode('utf-8') if vs.PY3 else result
        df = pd.read_csv(StringIO(result))
        return df
    else:
        print(result)
        return None    
    


#!/usr/bin/env python
# -*- coding:utf-8 -*- 

'''
Created on 2016年9月25日
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
'''
import six
import pandas as pd
import requests
import time
from threading import Thread
from tushare.trader import vars as vs
from tushare.trader import utils
from tushare.util import upass as up

class TraderAPI(object):
    """
    股票实盘交易接口
    提醒：本文涉及的思路和内容仅限于量化投资及程序化交易的研究与尝试，不作为个人或机构常规程序化交易的依据，
    不对实盘的交易风险和政策风险产生的影响负责，如有问题请与我联系。
    投资有风险，下单须谨慎。
    """
    def __init__(self, broker = ''):
        if broker == '':
            return None
        self.broker = broker
        self.trade_prefix = vs.CSC_PREFIX % (vs.P_TYPE['https'], 
                                             vs.DOMAINS['csc'],
                                             vs.PAGES['csclogin'])
        self.heart_active = True
        self.s = requests.session()
        if six.PY2:
            self.heart_thread = Thread(target = self.send_heartbeat)
            self.heart_thread.setDaemon(True)
        else:
            self.heart_thread = Thread(target = self.send_heartbeat, 
                                       daemon=True)
            
            
    def login(self):
        self.s.headers.update(vs.AGENT)
        self.s.get(vs.CSC_PREFIX % (vs.P_TYPE['https'], vs.DOMAINS['csc'],
                                             vs.PAGES['csclogin']))
        res = self.s.get(vs.V_CODE_URL%(vs.P_TYPE['https'],
                                          vs.DOMAINS['csc'],
                                          vs.PAGES['vimg']))
        if self._login(utils.get_vcode('csc', res)) is False:
            print('请确认账号或密码是否正确 ，或券商服务器是否处于维护中。 ')
        self.keepalive()
        
        
    def _login(self, v_code):
        brokerinfo = up.get_broker(self.broker)
        user = brokerinfo['user'][0]
        login_params = dict(
            inputid = user,                                 
            j_username = user,
            j_inputid = user,
            AppendCode = v_code,
            isCheckAppendCode = 'false',
            logined = 'false',
            f_tdx = '',
            j_cpu = '',
            j_password = brokerinfo['passwd'][0]
        )
        logined = self.s.post(vs.CSC_LOGIN_ACTION % (vs.P_TYPE['https'], 
                                                     vs.DOMAINS['csc']), 
                              params = login_params)
        if logined.text.find(u'消息中心') != -1:
            return True
        return False
    
    
    def keepalive(self):
        if self.heart_thread.is_alive():
            self.heart_active = True
        else:
            self.heart_thread.start()


    def send_heartbeat(self):
        while True:
            if self.heart_active:
                try:
                    response = self.heartbeat()
                    self.check_account_live(response)
                except:
                    self.login()
                time.sleep(100)
            else:
                time.sleep(10)


    def heartbeat(self):
        return self.baseinfo


    def exit(self):
        self.heart_active = False

    
    def buy(self, stkcode, price=0, count=0, amount=0):
        """
    买入证券
        params
        ---------
        stkcode:股票代码，string
        pricce:委托价格，int
        count:买入数量
        amount:买入金额
        """
        jsonobj = utils.get_jdata(self._trading(stkcode, price, 
                                                count, amount, 'B', 'buy'))
        res = True if jsonobj['result'] == 'true' else False
        return res
        
    
    def sell(self, stkcode, price=0, count=0, amount=0):
        """
    卖出证券
        params
        ---------
        stkcode:股票代码，string
        pricce:委托价格，int
        count:卖出数量
        amount:卖出金额
        """
        jsonobj = utils.get_jdata(self._trading(stkcode, price, count, 
                                                amount, 'S', 'sell'))
        res = True if jsonobj['result'] == 'true' else False
        return res
    
    
    def _trading(self, stkcode, price, count, amount, tradeflag, tradetype):
        txtdata = self.s.get(vs.TRADE_CHECK_URL % (vs.P_TYPE['https'], 
                                                   vs.DOMAINS['csc'], 
                                                   vs.PAGES['tradecheck'],
                                                   tradeflag, stkcode, 
                                                   tradetype, utils.nowtime_str()))
        jsonobj = utils.get_jdata(txtdata)
        list = jsonobj['returnList'][0]
        secuid = list['buysSecuid']
        fundavl = list['fundavl']
        stkname = list['stkname']
        if secuid is not None:
            if tradeflag == 'B':
                buytype = vs.BUY
                count = count if count else amount // price // 100 * 100 
            else:
                buytype = vs.SELL
                count = count if count else amount // price
                
            tradeparams = dict(
            stkname = stkname,
            stkcode = stkcode,
            secuid = secuid,
            buytype = buytype,
            bsflag = tradeflag,
            maxstkqty = '',
            buycount = count,
            buyprice = price,
            _ = utils.nowtime_str()
            )
            tradeResult = self.s.post(vs.TRADE_URL % (vs.P_TYPE['https'], 
                                                      vs.DOMAINS['csc'], 
                                                      vs.PAGES['trade']), 
                        params = tradeparams)
            return tradeResult
        return None
        

    def position(self):
        """
    获取持仓列表
        return:DataFrame
        ----------------------
        stkcode:证券代码
        stkname:证券名称
        stkqty :证券数量
        stkavl :可用数量
        lastprice:最新价格
        costprice:成本价
        income :参考盈亏（元）
        """
        return self._get_position()
    
    
    def _get_position(self):
        self.s.headers.update(vs.AGENT)
        txtdata = self.s.get(vs.BASE_URL % (vs.P_TYPE['https'], 
                                            vs.DOMAINS['csc'], 
                                            vs.PAGES['position']))
        jsonobj = utils.get_jdata(txtdata)
        df = pd.DataFrame(jsonobj['data'], columns=vs.POSITION_COLS)
        return df
    
    
    def entrust_list(self):
        """
       获取委托单列表
       return:DataFrame
       ----------
       ordersno:委托单号
       stkcode:证券代码
       stkname:证券名称
       bsflagState:买卖标志
       orderqty:委托数量
       matchqty:成交数量
       orderprice:委托价格
       operdate:交易日期
       opertime:交易时间
       orderdate:下单日期
       state:状态
        """
        txtdata = self.s.get(vs.ENTRUST_LIST_URL % (vs.P_TYPE['https'], 
                                                    vs.DOMAINS['csc'], 
                                                    vs.PAGES['entrustlist'],
                                            utils.nowtime_str()))
        jsonobj = utils.get_jdata(txtdata)
        df = pd.DataFrame(jsonobj['data'], columns=vs.ENTRUST_LIST_COLS)
        return df
    
    
    def deal_list(self, begin=None, end=None):
        """
    获取成交列表
        params
        -----------
        begin:开始日期  YYYYMMDD
        end:结束日期  YYYYMMDD
        
        return: DataFrame
        -----------
        ordersno:委托单号
        matchcode:成交编号
        trddate:交易日期
        matchtime:交易时间
        stkcode:证券代码
        stkname:证券名称
        bsflagState:买卖标志
        orderprice:委托价格
        matchprice:成交价格
        orderqty:委托数量
        matchqty:成交数量
        matchamt:成交金额
        """
        daterange = ''
        if (begin is None) & (end is None):
            selecttype = 'intraDay'
        else:
            daterange = vs.DEAL_DATE_RANGE % (begin, end)
            selecttype = 'all'
        txtdata = self.s.get(vs.DEAL_LIST_URL % (vs.P_TYPE['https'], 
                                                 vs.DOMAINS['csc'], 
                                                 vs.PAGES['deallist'],
                                                 selecttype, daterange, 
                                                 utils.nowtime_str()))
        jsonobj = utils.get_jdata(txtdata)
        df = pd.DataFrame(jsonobj['data'], columns=vs.DEAL_LIST_COLS)
        return df
    
    
    def cancel(self, ordersno='', orderdate=''):
        """
                 撤单
        params
        -----------
        ordersno:委托单号，多个以逗号分隔，e.g. 1866,1867
        orderdata:委托日期 YYYYMMDD，多个以逗号分隔，对应委托单好
        return
        ------------
        string
        """
        if (ordersno != '') & (orderdate != ''):
            params = dict(
                  ordersno = ordersno,
                  orderdate = orderdate,
                  _ = utils.nowtime_str()             
                               
            )
            result = self.s.post(vs.CANCEL_URL % (vs.P_TYPE['https'], vs.DOMAINS['csc'], vs.PAGES['cancel']), 
                            params = params)
            jsonobj = utils.get_jdata(result.text)  
            return jsonobj['msgMap']['ResultSucess']
        return None
    
    
    def baseinfo(self):
        """
    获取帐户基本信息
        return: Series
        -------------
        fundid:帐户ID
        gpsz: 股票市值
        fundvalue:基金市值
        jihelicai:集合理财
        fundbal:帐户余额
        marketvalue:总资产
        fundavl:可用余额
        daixiao:代销份额
        otc:OTC份额
        """
        return self._get_baseinfo()
    
    def _get_baseinfo(self):
        self.s.headers.update(vs.AGENT)
        txtdata = self.s.get(vs.BASE_URL % (vs.P_TYPE['https'], vs.DOMAINS['csc'], vs.PAGES['baseInfo']))
        jsonobj = utils.get_jdata(txtdata)
        stkdata = jsonobj['data']['moneytype0']
        stkdata['fundid'] = jsonobj['fundid']
        return pd.Series(stkdata)
    

    def check_login_status(self, return_data):
        if hasattr(return_data, 'get') and return_data.get('error_no') == '-1':
            raise NotLoginError
        
        
class NotLoginError(Exception):
    def __init__(self, result=None):
        super(NotLoginError, self).__init__()
        self.result = result
    def heartbeat(self):
        return self.baseinfo




# -*- coding:utf-8 -*- 
"""
通联数据
Created on 2015/08/24
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""

from pandas.compat import StringIO
import pandas as pd
from tushare.util import vars as vs
from tushare.util.common import Client
from tushare.util import upass as up

class Idx():
    
    def __init__(self, client=None):
        if client is None:
            self.client = Client(up.get_token())
        else:
            self.client = client
            
            
    def Idx(self, secID='', ticker='', field=''):
        """
            获取国内外指数的基本要素信息，包括指数名称、指数代码、发布机构、发布日期、基日、基点等。
        """
        code, result = self.client.getData(vs.IDX%(secID, ticker, field))
        return _ret_data(code, result)


    def IdxCons(self, secID='', ticker='', intoDate='', isNew='', field=''):
        """
            获取国内外指数的成分构成情况，包括指数成分股名称、成分股代码、入选日期、剔除日期等。
        """
        code, result = self.client.getData(vs.IDXCONS%(secID, ticker, intoDate, 
                                                       intoDate, isNew, field))
        return _ret_data(code, result)


def _ret_data(code, result):
    if code==200:
        result = result.decode('utf-8') if vs.PY3 else result
        df = pd.read_csv(StringIO(result))
        return df
    else:
        print(result)
        return None    
    
